@@ -187,3 +187,17 @@ type VolumeCreateBody struct {
 	// Required: true
 	Name string `json:"Name"`
 }
+
+// VolumeMountReport describes the response from volume mount
+type VolumeMountReport struct {
+	Err  error
+	Id   string //nolint
+	Name string
+	Path string
+}
+
+// VolumeUnmountReport describes the response from umounting a volume
+type VolumeUnmountReport struct {
+	Err error
+	Id  string //nolint
+}
