@@ -5,6 +5,8 @@ import (
 	"fmt"
 	"io/ioutil"
 	"math/rand"
+	"net"
+	"net/url"
 	"os"
 	"os/exec"
 	"path/filepath"
@@ -15,16 +17,16 @@ import (
 	"testing"
 	"time"
 
-	"github.com/containers/podman/v3/libpod/define"
-	"github.com/containers/podman/v3/pkg/cgroups"
-	"github.com/containers/podman/v3/pkg/inspect"
-	"github.com/containers/podman/v3/pkg/rootless"
-	. "github.com/containers/podman/v3/test/utils"
+	"github.com/containers/common/pkg/cgroups"
+	"github.com/containers/podman/v4/libpod/define"
+	"github.com/containers/podman/v4/pkg/inspect"
+	"github.com/containers/podman/v4/pkg/rootless"
+	"github.com/containers/podman/v4/pkg/util"
+	. "github.com/containers/podman/v4/test/utils"
 	"github.com/containers/storage"
 	"github.com/containers/storage/pkg/reexec"
 	"github.com/containers/storage/pkg/stringid"
 	jsoniter "github.com/json-iterator/go"
-	"github.com/onsi/ginkgo"
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
 	. "github.com/onsi/gomega/gexec"
@@ -33,24 +35,21 @@ import (
 )
 
 var (
-	PODMAN_BINARY      string
-	CONMON_BINARY      string
-	CNI_CONFIG_DIR     string
-	RUNC_BINARY        string
-	INTEGRATION_ROOT   string
-	CGROUP_MANAGER     = "systemd"
-	ARTIFACT_DIR       = "/tmp/.artifacts"
-	RESTORE_IMAGES     = []string{ALPINE, BB, nginx}
+	//lint:ignore ST1003
+	PODMAN_BINARY      string                        //nolint:revive,stylecheck
+	INTEGRATION_ROOT   string                        //nolint:revive,stylecheck
+	CGROUP_MANAGER     = "systemd"                   //nolint:revive,stylecheck
+	RESTORE_IMAGES     = []string{ALPINE, BB, nginx} //nolint:revive,stylecheck
 	defaultWaitTimeout = 90
-	CGROUPSV2, _       = cgroups.IsCgroup2UnifiedMode()
+	CGROUPSV2, _       = cgroups.IsCgroup2UnifiedMode() //nolint:revive,stylecheck
 )
 
 // PodmanTestIntegration struct for command line options
 type PodmanTestIntegration struct {
 	PodmanTest
 	ConmonBinary        string
-	CrioRoot            string
-	CNIConfigDir        string
+	Root                string
+	NetworkConfigDir    string
 	OCIRuntime          string
 	RunRoot             string
 	StorageOptions      string
@@ -74,8 +73,6 @@ type testResult struct {
 	length float64
 }
 
-var noCache = "Cannot run nocache with remote"
-
 type testResultsSorted []testResult
 
 func (a testResultsSorted) Len() int      { return len(a) }
@@ -116,13 +113,6 @@ var _ = SynchronizedBeforeSuite(func() []byte {
 	cwd, _ := os.Getwd()
 	INTEGRATION_ROOT = filepath.Join(cwd, "../../")
 	podman := PodmanTestSetup("/tmp")
-	podman.ArtifactPath = ARTIFACT_DIR
-	if _, err := os.Stat(ARTIFACT_DIR); os.IsNotExist(err) {
-		if err = os.Mkdir(ARTIFACT_DIR, 0777); err != nil {
-			fmt.Printf("%q\n", err)
-			os.Exit(1)
-		}
-	}
 
 	// Pull cirros but don't put it into the cache
 	pullImages := []string{cirros, fedoraToolbox, volumeTest}
@@ -135,7 +125,7 @@ var _ = SynchronizedBeforeSuite(func() []byte {
 		fmt.Printf("%q\n", err)
 		os.Exit(1)
 	}
-	podman.CrioRoot = ImageCacheDir
+	podman.Root = ImageCacheDir
 	// If running localized tests, the cache dir is created and populated. if the
 	// tests are remote, this is a no-op
 	populateCache(podman)
@@ -175,7 +165,6 @@ var _ = SynchronizedBeforeSuite(func() []byte {
 func (p *PodmanTestIntegration) Setup() {
 	cwd, _ := os.Getwd()
 	INTEGRATION_ROOT = filepath.Join(cwd, "../../")
-	p.ArtifactPath = ARTIFACT_DIR
 }
 
 var _ = SynchronizedAfterSuite(func() {},
@@ -186,14 +175,14 @@ var _ = SynchronizedAfterSuite(func() {},
 			fmt.Printf("%s\t\t%f\n", result.name, result.length)
 		}
 
-		// previous crio-run
+		// previous runroot
 		tempdir, err := CreateTempDirInTempDir()
 		if err != nil {
 			os.Exit(1)
 		}
 		podmanTest := PodmanTestCreate(tempdir)
 
-		if err := os.RemoveAll(podmanTest.CrioRoot); err != nil {
+		if err := os.RemoveAll(podmanTest.Root); err != nil {
 			fmt.Printf("%q\n", err)
 		}
 
@@ -208,25 +197,23 @@ var _ = SynchronizedAfterSuite(func() {},
 
 // PodmanTestCreate creates a PodmanTestIntegration instance for the tests
 func PodmanTestCreateUtil(tempDir string, remote bool) *PodmanTestIntegration {
-	var (
-		podmanRemoteBinary string
-	)
+	var podmanRemoteBinary string
 
 	host := GetHostDistributionInfo()
 	cwd, _ := os.Getwd()
 
+	root := filepath.Join(tempDir, "root")
 	podmanBinary := filepath.Join(cwd, "../../bin/podman")
 	if os.Getenv("PODMAN_BINARY") != "" {
 		podmanBinary = os.Getenv("PODMAN_BINARY")
 	}
 
-	if remote {
-		podmanRemoteBinary = filepath.Join(cwd, "../../bin/podman-remote")
-		if os.Getenv("PODMAN_REMOTE_BINARY") != "" {
-			podmanRemoteBinary = os.Getenv("PODMAN_REMOTE_BINARY")
-		}
+	podmanRemoteBinary = filepath.Join(cwd, "../../bin/podman-remote")
+	if os.Getenv("PODMAN_REMOTE_BINARY") != "" {
+		podmanRemoteBinary = os.Getenv("PODMAN_REMOTE_BINARY")
 	}
-	conmonBinary := filepath.Join("/usr/libexec/podman/conmon")
+
+	conmonBinary := "/usr/libexec/podman/conmon"
 	altConmonBinary := "/usr/bin/conmon"
 	if _, err := os.Stat(conmonBinary); os.IsNotExist(err) {
 		conmonBinary = altConmonBinary
@@ -252,11 +239,26 @@ func PodmanTestCreateUtil(tempDir string, remote bool) *PodmanTestIntegration {
 		ociRuntime = "crun"
 	}
 	os.Setenv("DISABLE_HC_SYSTEMD", "true")
-	CNIConfigDir := "/etc/cni/net.d"
+
+	networkBackend := CNI
+	networkConfigDir := "/etc/cni/net.d"
 	if rootless.IsRootless() {
-		CNIConfigDir = filepath.Join(os.Getenv("HOME"), ".config/cni/net.d")
+		networkConfigDir = filepath.Join(os.Getenv("HOME"), ".config/cni/net.d")
 	}
-	if err := os.MkdirAll(CNIConfigDir, 0755); err != nil {
+
+	if strings.ToLower(os.Getenv("NETWORK_BACKEND")) == "netavark" {
+		networkBackend = Netavark
+		networkConfigDir = "/etc/containers/networks"
+		if rootless.IsRootless() {
+			networkConfigDir = filepath.Join(root, "etc", "networks")
+		}
+	}
+
+	if err := os.MkdirAll(root, 0755); err != nil {
+		panic(err)
+	}
+
+	if err := os.MkdirAll(networkConfigDir, 0755); err != nil {
 		panic(err)
 	}
 
@@ -264,36 +266,59 @@ func PodmanTestCreateUtil(tempDir string, remote bool) *PodmanTestIntegration {
 	if rootless.IsRootless() {
 		storageFs = ROOTLESS_STORAGE_FS
 	}
+	if os.Getenv("STORAGE_FS") != "" {
+		storageFs = os.Getenv("STORAGE_FS")
+		storageOptions = "--storage-driver " + storageFs
+	}
 	p := &PodmanTestIntegration{
 		PodmanTest: PodmanTest{
-			PodmanBinary:  podmanBinary,
-			ArtifactPath:  ARTIFACT_DIR,
-			TempDir:       tempDir,
-			RemoteTest:    remote,
-			ImageCacheFS:  storageFs,
-			ImageCacheDir: ImageCacheDir,
+			PodmanBinary:       podmanBinary,
+			RemotePodmanBinary: podmanRemoteBinary,
+			TempDir:            tempDir,
+			RemoteTest:         remote,
+			ImageCacheFS:       storageFs,
+			ImageCacheDir:      ImageCacheDir,
+			NetworkBackend:     networkBackend,
 		},
 		ConmonBinary:        conmonBinary,
-		CrioRoot:            filepath.Join(tempDir, "crio"),
+		Root:                root,
 		TmpDir:              tempDir,
-		CNIConfigDir:        CNIConfigDir,
+		NetworkConfigDir:    networkConfigDir,
 		OCIRuntime:          ociRuntime,
-		RunRoot:             filepath.Join(tempDir, "crio-run"),
+		RunRoot:             filepath.Join(tempDir, "runroot"),
 		StorageOptions:      storageOptions,
 		SignaturePolicyPath: filepath.Join(INTEGRATION_ROOT, "test/policy.json"),
 		CgroupManager:       cgroupManager,
 		Host:                host,
 	}
+
 	if remote {
-		p.PodmanTest.RemotePodmanBinary = podmanRemoteBinary
-		uuid := stringid.GenerateNonCryptoID()
+		var pathPrefix string
 		if !rootless.IsRootless() {
-			p.RemoteSocket = fmt.Sprintf("unix:/run/podman/podman-%s.sock", uuid)
+			pathPrefix = "/run/podman/podman"
 		} else {
 			runtimeDir := os.Getenv("XDG_RUNTIME_DIR")
-			socket := fmt.Sprintf("podman-%s.sock", uuid)
-			fqpath := filepath.Join(runtimeDir, socket)
-			p.RemoteSocket = fmt.Sprintf("unix:%s", fqpath)
+			pathPrefix = filepath.Join(runtimeDir, "podman")
+		}
+		// We want to avoid collisions in socket paths, but using the
+		// socket directly for a collision check doesn’t work; bind(2) on AF_UNIX
+		// creates the file, and we need to pass a unique path now before the bind(2)
+		// happens. So, use a podman-%s.sock-lock empty file as a marker.
+		tries := 0
+		for {
+			uuid := stringid.GenerateNonCryptoID()
+			lockPath := fmt.Sprintf("%s-%s.sock-lock", pathPrefix, uuid)
+			lockFile, err := os.OpenFile(lockPath, os.O_WRONLY|os.O_CREATE|os.O_EXCL, 0700)
+			if err == nil {
+				lockFile.Close()
+				p.RemoteSocketLock = lockPath
+				p.RemoteSocket = fmt.Sprintf("unix:%s-%s.sock", pathPrefix, uuid)
+				break
+			}
+			tries++
+			if tries >= 1000 {
+				panic("Too many RemoteSocket collisions")
+			}
 		}
 	}
 
@@ -306,29 +331,43 @@ func PodmanTestCreateUtil(tempDir string, remote bool) *PodmanTestIntegration {
 
 func (p PodmanTestIntegration) AddImageToRWStore(image string) {
 	if err := p.RestoreArtifact(image); err != nil {
-		logrus.Errorf("unable to restore %s to RW store", image)
+		logrus.Errorf("Unable to restore %s to RW store", image)
 	}
 }
 
-// createArtifact creates a cached image in the artifact dir
+func imageTarPath(image string) string {
+	cacheDir := os.Getenv("PODMAN_TEST_IMAGE_CACHE_DIR")
+	if cacheDir == "" {
+		cacheDir = os.Getenv("TMPDIR")
+		if cacheDir == "" {
+			cacheDir = "/tmp"
+		}
+	}
+
+	// e.g., registry.com/fubar:latest -> registry.com-fubar-latest.tar
+	imageCacheName := strings.ReplaceAll(strings.ReplaceAll(image, ":", "-"), "/", "-") + ".tar"
+
+	return filepath.Join(cacheDir, imageCacheName)
+}
+
+// createArtifact creates a cached image tarball in a local directory
 func (p *PodmanTestIntegration) createArtifact(image string) {
 	if os.Getenv("NO_TEST_CACHE") != "" {
 		return
 	}
-	dest := strings.Split(image, "/")
-	destName := fmt.Sprintf("/tmp/%s.tar", strings.Replace(strings.Join(strings.Split(dest[len(dest)-1], "/"), ""), ":", "-", -1))
-	fmt.Printf("Caching %s at %s...", image, destName)
+	destName := imageTarPath(image)
 	if _, err := os.Stat(destName); os.IsNotExist(err) {
+		fmt.Printf("Caching %s at %s...\n", image, destName)
 		pull := p.PodmanNoCache([]string{"pull", image})
 		pull.Wait(440)
-		Expect(pull.ExitCode()).To(Equal(0))
+		Expect(pull).Should(Exit(0))
 
 		save := p.PodmanNoCache([]string{"save", "-o", destName, image})
 		save.Wait(90)
-		Expect(save.ExitCode()).To(Equal(0))
+		Expect(save).Should(Exit(0))
 		fmt.Printf("\n")
 	} else {
-		fmt.Printf(" already exists.\n")
+		fmt.Printf("[image already cached: %s]\n", destName)
 	}
 }
 
@@ -424,7 +463,8 @@ func (p *PodmanTestIntegration) RunNginxWithHealthCheck(name string) (*PodmanSes
 	if name != "" {
 		podmanArgs = append(podmanArgs, "--name", name)
 	}
-	podmanArgs = append(podmanArgs, "-dt", "-P", "--health-cmd", "curl http://localhost/", nginx)
+	// curl without -f exits 0 even if http code >= 400!
+	podmanArgs = append(podmanArgs, "-dt", "-P", "--health-cmd", "curl -f http://localhost/", nginx)
 	session := p.Podman(podmanArgs)
 	session.WaitWithDefaultTimeout()
 	return session, session.OutputToString()
@@ -451,31 +491,26 @@ func (p *PodmanTestIntegration) RunLsContainerInPod(name, pod string) (*PodmanSe
 // BuildImage uses podman build and buildah to build an image
 // called imageName based on a string dockerfile
 func (p *PodmanTestIntegration) BuildImage(dockerfile, imageName string, layers string) string {
-	dockerfilePath := filepath.Join(p.TempDir, "Dockerfile")
-	err := ioutil.WriteFile(dockerfilePath, []byte(dockerfile), 0755)
-	Expect(err).To(BeNil())
-	cmd := []string{"build", "--pull-never", "--layers=" + layers, "--file", dockerfilePath}
-	if len(imageName) > 0 {
-		cmd = append(cmd, []string{"-t", imageName}...)
-	}
-	cmd = append(cmd, p.TempDir)
-	session := p.Podman(cmd)
-	session.Wait(240)
-	Expect(session).Should(Exit(0), fmt.Sprintf("BuildImage session output: %q", session.OutputToString()))
-	output := session.OutputToStringArray()
-	return output[len(output)-1]
+	return p.buildImage(dockerfile, imageName, layers, "")
+}
+
+// BuildImageWithLabel uses podman build and buildah to build an image
+// called imageName based on a string dockerfile, adds desired label to paramset
+func (p *PodmanTestIntegration) BuildImageWithLabel(dockerfile, imageName string, layers string, label string) string {
+	return p.buildImage(dockerfile, imageName, layers, label)
 }
 
 // PodmanPID execs podman and returns its PID
 func (p *PodmanTestIntegration) PodmanPID(args []string) (*PodmanSessionIntegration, int) {
 	podmanOptions := p.MakeOptions(args, false, false)
 	fmt.Printf("Running: %s %s\n", p.PodmanBinary, strings.Join(podmanOptions, " "))
+
 	command := exec.Command(p.PodmanBinary, podmanOptions...)
 	session, err := Start(command, GinkgoWriter, GinkgoWriter)
 	if err != nil {
-		Fail(fmt.Sprintf("unable to run podman command: %s", strings.Join(podmanOptions, " ")))
+		Fail("unable to run podman command: " + strings.Join(podmanOptions, " "))
 	}
-	podmanSession := &PodmanSession{session}
+	podmanSession := &PodmanSession{Session: session}
 	return &PodmanSessionIntegration{podmanSession}, command.Process.Pid
 }
 
@@ -597,9 +632,9 @@ func (p *PodmanTestIntegration) RunHealthCheck(cid string) error {
 	return errors.Errorf("unable to detect %s as running", cid)
 }
 
-func (p *PodmanTestIntegration) CreateSeccompJson(in []byte) (string, error) {
+func (p *PodmanTestIntegration) CreateSeccompJSON(in []byte) (string, error) {
 	jsonFile := filepath.Join(p.TempDir, "seccomp.json")
-	err := WriteJsonFile(in, jsonFile)
+	err := WriteJSONFile(in, jsonFile)
 	if err != nil {
 		return "", err
 	}
@@ -622,28 +657,41 @@ func SkipIfRootlessCgroupsV1(reason string) {
 func SkipIfRootless(reason string) {
 	checkReason(reason)
 	if os.Geteuid() != 0 {
-		ginkgo.Skip("[rootless]: " + reason)
+		Skip("[rootless]: " + reason)
 	}
 }
 
 func SkipIfNotRootless(reason string) {
 	checkReason(reason)
 	if os.Geteuid() == 0 {
-		ginkgo.Skip("[notRootless]: " + reason)
+		Skip("[notRootless]: " + reason)
+	}
+}
+
+func SkipIfSystemdNotRunning(reason string) {
+	checkReason(reason)
+
+	cmd := exec.Command("systemctl", "list-units")
+	err := cmd.Run()
+	if err != nil {
+		if _, ok := err.(*exec.Error); ok {
+			Skip("[notSystemd]: not running " + reason)
+		}
+		Expect(err).ToNot(HaveOccurred())
 	}
 }
 
 func SkipIfNotSystemd(manager, reason string) {
 	checkReason(reason)
 	if manager != "systemd" {
-		ginkgo.Skip("[notSystemd]: " + reason)
+		Skip("[notSystemd]: " + reason)
 	}
 }
 
 func SkipIfNotFedora() {
 	info := GetHostDistributionInfo()
 	if info.Distribution != "fedora" {
-		ginkgo.Skip("Test can only run on Fedora")
+		Skip("Test can only run on Fedora")
 	}
 }
 
@@ -651,9 +699,13 @@ func isRootless() bool {
 	return os.Geteuid() != 0
 }
 
+func isCgroupsV1() bool {
+	return !CGROUPSV2
+}
+
 func SkipIfCgroupV1(reason string) {
 	checkReason(reason)
-	if !CGROUPSV2 {
+	if isCgroupsV1() {
 		Skip(reason)
 	}
 }
@@ -667,10 +719,7 @@ func SkipIfCgroupV2(reason string) {
 
 func isContainerized() bool {
 	// This is set to "podman" by podman automatically
-	if os.Getenv("container") != "" {
-		return true
-	}
-	return false
+	return os.Getenv("container") != ""
 }
 
 func SkipIfContainerized(reason string) {
@@ -680,9 +729,64 @@ func SkipIfContainerized(reason string) {
 	}
 }
 
+func SkipIfRemote(reason string) {
+	checkReason(reason)
+	if !IsRemote() {
+		return
+	}
+	Skip("[remote]: " + reason)
+}
+
+func SkipIfNotRemote(reason string) {
+	checkReason(reason)
+	if IsRemote() {
+		return
+	}
+	Skip("[local]: " + reason)
+}
+
+// SkipIfInContainer skips a test if the test is run inside a container
+func SkipIfInContainer(reason string) {
+	checkReason(reason)
+	if os.Getenv("TEST_ENVIRON") == "container" {
+		Skip("[container]: " + reason)
+	}
+}
+
+// SkipIfNotActive skips a test if the given systemd unit is not active
+func SkipIfNotActive(unit string, reason string) {
+	checkReason(reason)
+
+	var buffer bytes.Buffer
+	cmd := exec.Command("systemctl", "is-active", unit)
+	cmd.Stdout = &buffer
+	err := cmd.Start()
+	Expect(err).ToNot(HaveOccurred())
+
+	err = cmd.Wait()
+	Expect(err).ToNot(HaveOccurred())
+
+	Expect(err).ToNot(HaveOccurred())
+	if strings.TrimSpace(buffer.String()) != "active" {
+		Skip(fmt.Sprintf("[systemd]: unit %s is not active: %s", unit, reason))
+	}
+}
+
+func SkipIfCNI(p *PodmanTestIntegration) {
+	if p.NetworkBackend == CNI {
+		Skip("this test is not compatible with the CNI network backend")
+	}
+}
+
+func SkipIfNetavark(p *PodmanTestIntegration) {
+	if p.NetworkBackend == Netavark {
+		Skip("This test is not compatible with the netavark network backend")
+	}
+}
+
 // PodmanAsUser is the exec call to podman on the filesystem with the specified uid/gid and environment
 func (p *PodmanTestIntegration) PodmanAsUser(args []string, uid, gid uint32, cwd string, env []string) *PodmanSessionIntegration {
-	podmanSession := p.PodmanAsUserBase(args, uid, gid, cwd, env, false, false, nil)
+	podmanSession := p.PodmanAsUserBase(args, uid, gid, cwd, env, false, false, nil, nil)
 	return &PodmanSessionIntegration{podmanSession}
 }
 
@@ -694,18 +798,20 @@ func (p *PodmanTestIntegration) RestartRemoteService() {
 
 // RestoreArtifactToCache populates the imagecache from tarballs that were cached earlier
 func (p *PodmanTestIntegration) RestoreArtifactToCache(image string) error {
-	fmt.Printf("Restoring %s...\n", image)
-	dest := strings.Split(image, "/")
-	destName := fmt.Sprintf("/tmp/%s.tar", strings.Replace(strings.Join(strings.Split(dest[len(dest)-1], "/"), ""), ":", "-", -1))
-	p.CrioRoot = p.ImageCacheDir
-	restore := p.PodmanNoEvents([]string{"load", "-q", "-i", destName})
-	restore.WaitWithDefaultTimeout()
+	tarball := imageTarPath(image)
+	if _, err := os.Stat(tarball); err == nil {
+		fmt.Printf("Restoring %s...\n", image)
+		p.Root = p.ImageCacheDir
+		restore := p.PodmanNoEvents([]string{"load", "-q", "-i", tarball})
+		restore.WaitWithDefaultTimeout()
+	}
 	return nil
 }
 
 func populateCache(podman *PodmanTestIntegration) {
 	for _, image := range CACHE_IMAGES {
-		podman.RestoreArtifactToCache(image)
+		err := podman.RestoreArtifactToCache(image)
+		Expect(err).To(BeNil())
 	}
 	// logformatter uses this to recognize the first test
 	fmt.Printf("-----------------------------\n")
@@ -738,10 +844,14 @@ func (p *PodmanTestIntegration) PodmanNoEvents(args []string) *PodmanSessionInte
 // MakeOptions assembles all the podman main options
 func (p *PodmanTestIntegration) makeOptions(args []string, noEvents, noCache bool) []string {
 	if p.RemoteTest {
+		if !util.StringInSlice("--remote", args) {
+			return append([]string{"--remote", "--url", p.RemoteSocket}, args...)
+		}
 		return args
 	}
+
 	var debug string
-	if _, ok := os.LookupEnv("DEBUG"); ok {
+	if _, ok := os.LookupEnv("E2E_DEBUG"); ok {
 		debug = "--log-level=debug --syslog=true "
 	}
 
@@ -750,10 +860,11 @@ func (p *PodmanTestIntegration) makeOptions(args []string, noEvents, noCache boo
 		eventsType = "none"
 	}
 
-	podmanOptions := strings.Split(fmt.Sprintf("%s--root %s --runroot %s --runtime %s --conmon %s --cni-config-dir %s --cgroup-manager %s --tmpdir %s --events-backend %s",
-		debug, p.CrioRoot, p.RunRoot, p.OCIRuntime, p.ConmonBinary, p.CNIConfigDir, p.CgroupManager, p.TmpDir, eventsType), " ")
-	if os.Getenv("HOOK_OPTION") != "" {
-		podmanOptions = append(podmanOptions, os.Getenv("HOOK_OPTION"))
+	podmanOptions := strings.Split(fmt.Sprintf("%s--root %s --runroot %s --runtime %s --conmon %s --network-config-dir %s --cgroup-manager %s --tmpdir %s --events-backend %s",
+		debug, p.Root, p.RunRoot, p.OCIRuntime, p.ConmonBinary, p.NetworkConfigDir, p.CgroupManager, p.TmpDir, eventsType), " ")
+
+	if !p.RemoteTest {
+		podmanOptions = append(podmanOptions, "--network-backend", p.NetworkBackend.ToString())
 	}
 
 	podmanOptions = append(podmanOptions, strings.Split(p.StorageOptions, " ")...)
@@ -767,7 +878,12 @@ func (p *PodmanTestIntegration) makeOptions(args []string, noEvents, noCache boo
 }
 
 func writeConf(conf []byte, confPath string) {
-	if err := ioutil.WriteFile(confPath, conf, 777); err != nil {
+	if _, err := os.Stat(filepath.Dir(confPath)); os.IsNotExist(err) {
+		if err := os.MkdirAll(filepath.Dir(confPath), 0o777); err != nil {
+			fmt.Println(err)
+		}
+	}
+	if err := ioutil.WriteFile(confPath, conf, 0o777); err != nil {
 		fmt.Println(err)
 	}
 }
@@ -778,13 +894,18 @@ func removeConf(confPath string) {
 	}
 }
 
-// generateNetworkConfig generates a cni config with a random name
+// generateNetworkConfig generates a CNI or Netavark config with a random name
 // it returns the network name and the filepath
 func generateNetworkConfig(p *PodmanTestIntegration) (string, string) {
+	var (
+		path string
+		conf string
+	)
 	// generate a random name to prevent conflicts with other tests
 	name := "net" + stringid.GenerateNonCryptoID()
-	path := filepath.Join(p.CNIConfigDir, fmt.Sprintf("%s.conflist", name))
-	conf := fmt.Sprintf(`{
+	if p.NetworkBackend != Netavark {
+		path = filepath.Join(p.NetworkConfigDir, fmt.Sprintf("%s.conflist", name))
+		conf = fmt.Sprintf(`{
 		"cniVersion": "0.3.0",
 		"name": "%s",
 		"plugins": [
@@ -809,22 +930,170 @@ func generateNetworkConfig(p *PodmanTestIntegration) (string, string) {
 		  }
 		]
 	}`, name)
+	} else {
+		path = filepath.Join(p.NetworkConfigDir, fmt.Sprintf("%s.json", name))
+		conf = fmt.Sprintf(`
+{
+     "name": "%s",
+     "id": "e1ef2749024b88f5663ca693a9118e036d6bfc48bcfe460faf45e9614a513e5c",
+     "driver": "bridge",
+     "network_interface": "netavark1",
+     "created": "2022-01-05T14:15:10.975493521-06:00",
+     "subnets": [
+          {
+               "subnet": "10.100.0.0/16",
+               "gateway": "10.100.0.1"
+          }
+     ],
+     "ipv6_enabled": false,
+     "internal": false,
+     "dns_enabled": true,
+     "ipam_options": {
+          "driver": "host-local"
+     }
+}
+`, name)
+	}
 	writeConf([]byte(conf), path)
-
 	return name, path
 }
 
-func (p *PodmanTestIntegration) removeCNINetwork(name string) {
+func (p *PodmanTestIntegration) removeNetwork(name string) {
 	session := p.Podman([]string{"network", "rm", "-f", name})
 	session.WaitWithDefaultTimeout()
-	Expect(session.ExitCode()).To(BeNumerically("<=", 1))
+	Expect(session.ExitCode()).To(BeNumerically("<=", 1), "Exit code must be 0 or 1")
 }
 
-func (p *PodmanSessionIntegration) jq(jqCommand string) (string, error) {
+func (s *PodmanSessionIntegration) jq(jqCommand string) (string, error) {
 	var out bytes.Buffer
 	cmd := exec.Command("jq", jqCommand)
-	cmd.Stdin = strings.NewReader(p.OutputToString())
+	cmd.Stdin = strings.NewReader(s.OutputToString())
 	cmd.Stdout = &out
 	err := cmd.Run()
 	return strings.TrimRight(out.String(), "\n"), err
 }
+
+func (p *PodmanTestIntegration) buildImage(dockerfile, imageName string, layers string, label string) string {
+	dockerfilePath := filepath.Join(p.TempDir, "Dockerfile")
+	err := ioutil.WriteFile(dockerfilePath, []byte(dockerfile), 0755)
+	Expect(err).To(BeNil())
+	cmd := []string{"build", "--pull-never", "--layers=" + layers, "--file", dockerfilePath}
+	if label != "" {
+		cmd = append(cmd, "--label="+label)
+	}
+	if len(imageName) > 0 {
+		cmd = append(cmd, []string{"-t", imageName}...)
+	}
+	cmd = append(cmd, p.TempDir)
+	session := p.Podman(cmd)
+	session.Wait(240)
+	Expect(session).Should(Exit(0), fmt.Sprintf("BuildImage session output: %q", session.OutputToString()))
+	output := session.OutputToStringArray()
+	return output[len(output)-1]
+}
+
+func writeYaml(content string, fileName string) error {
+	f, err := os.Create(fileName)
+	if err != nil {
+		return err
+	}
+	defer f.Close()
+
+	_, err = f.WriteString(content)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+// GetPort finds an unused port on the system
+func GetPort() int {
+	a, err := net.ResolveTCPAddr("tcp", "localhost:0")
+	if err != nil {
+		Fail(fmt.Sprintf("unable to get free port: %v", err))
+	}
+
+	l, err := net.ListenTCP("tcp", a)
+	if err != nil {
+		Fail(fmt.Sprintf("unable to get free port: %v", err))
+	}
+	defer l.Close()
+	return l.Addr().(*net.TCPAddr).Port
+}
+
+func ncz(port int) bool {
+	timeout := 500 * time.Millisecond
+	for i := 0; i < 5; i++ {
+		ncCmd := []string{"-z", "localhost", fmt.Sprintf("%d", port)}
+		fmt.Printf("Running: nc %s\n", strings.Join(ncCmd, " "))
+		check := SystemExec("nc", ncCmd)
+		if check.ExitCode() == 0 {
+			return true
+		}
+		time.Sleep(timeout)
+		timeout++
+	}
+	return false
+}
+
+func createNetworkName(name string) string {
+	return name + stringid.GenerateNonCryptoID()[:10]
+}
+
+var IPRegex = `(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)(\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)){3}`
+
+// digShort execs into the given container and does a dig lookup with a timeout
+// backoff.  If it gets a response, it ensures that the output is in the correct
+// format and iterates a string array for match
+func digShort(container, lookupName string, matchNames []string, p *PodmanTestIntegration) {
+	digInterval := time.Millisecond * 250
+	for i := 0; i < 6; i++ {
+		time.Sleep(digInterval * time.Duration(i))
+		dig := p.Podman([]string{"exec", container, "dig", "+short", lookupName})
+		dig.WaitWithDefaultTimeout()
+		if dig.ExitCode() == 0 {
+			output := dig.OutputToString()
+			Expect(output).To(MatchRegexp(IPRegex))
+			for _, name := range matchNames {
+				Expect(output).To(Equal(name))
+			}
+			// success
+			return
+		}
+	}
+	Fail("dns is not responding")
+}
+
+// WaitForFile to be created in defaultWaitTimeout seconds, returns false if file not created
+func WaitForFile(path string) (err error) {
+	until := time.Now().Add(time.Duration(defaultWaitTimeout) * time.Second)
+	for i := 1; time.Now().Before(until); i++ {
+		_, err = os.Stat(path)
+		switch {
+		case err == nil:
+			return nil
+		case errors.Is(err, os.ErrNotExist):
+			time.Sleep(time.Duration(i) * time.Second)
+		default:
+			return err
+		}
+	}
+	return err
+}
+
+// WaitForService blocks, waiting for some service listening on given host:port
+func WaitForService(address url.URL) {
+	// Wait for podman to be ready
+	var conn net.Conn
+	var err error
+	for i := 1; i <= 5; i++ {
+		conn, err = net.Dial("tcp", address.Host)
+		if err != nil {
+			// Podman not available yet...
+			time.Sleep(time.Duration(i) * time.Second)
+		}
+	}
+	Expect(err).ShouldNot(HaveOccurred())
+	conn.Close()
+}
